#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>
#include "Arduino.h"
#include <SoftwareSerial.h>

const byte rxPin = 9;
const byte txPin = 8;
SoftwareSerial BTSerial(rxPin, txPin); // RX TX
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

#define SERVOMIN  520 // this is the 'minimum' pulse length count (out of 4096)
#define SERVOMAX  140 // this is the 'maximum' pulse length count (out of 4096)

uint8_t servonum = 0;

int xval;
int yval;

int lexpulse;
int rexpulse;

int leypulse;
int reypulse;

int uplidpulse;
int lolidpulse;

int trimval;

const int analogInPin = A0;
int sensorValue = 0;
int outputValue = 0;
int switchval = 0;

const int blinkDuration = 200;  // Adjust this value for the desired blink duration in milliseconds

bool isBlinking = false;
unsigned long blinkStartTime = 0;

void setup() {
  Serial.begin(9600);
  Serial.println("8 channel Servo test!");
  pinMode(analogInPin, INPUT);
  pinMode(2, INPUT);
 pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);
  BTSerial.begin(9600);
  Serial.begin(9600);
  pwm.begin();
  pwm.setPWMFreq(60);  // Analog servos run at ~60 Hz updates

  delay(10);
}

String messageBuffer = "";
String message = "             ";

void setServoPulse(uint8_t n, double pulse) {
  double pulselength;
  
  pulselength = 1000000;   // 1,000,000 us per second
  pulselength /= 60;   // 60 Hz
  pulselength /= 4096;  // 12 bits of resolution
  pulse *= 1000000;  // convert to us
  pulse /= pulselength;
  pwm.setPWM(n, 0, pulse);
}

void blink() {
  // Close both upper and lower eyelids
  pwm.setPWM(4, 0, SERVOMIN); // Upper eyelid closed
  pwm.setPWM(5, 0, SERVOMIN); // Lower eyelid closed
}

void loop() {

  while (BTSerial.available() > 0) {
    char data = (char) BTSerial.read();
    messageBuffer += data;
    if (data == ';'){
      message = messageBuffer;
      messageBuffer = "";
      Serial.print(message); // send to serial monitor
      message = "You sent " + message;
      BTSerial.print(message); // send back to bluetooth terminal
    }
  }
  xval = analogRead(A1);
  lexpulse = map(xval, 0, 1023, 270, 390);
  rexpulse = lexpulse;

  switchval = digitalRead(2);

  yval = analogRead(A0);
  leypulse = map(yval, 0, 1023, 280, 400);
  reypulse = map(yval, 0, 1023, 400, 280);

  trimval = analogRead(A2);
  trimval = map(trimval, 320, 580, -40, 40);
  uplidpulse = map(yval, 0, 1023, 280, 420);
  uplidpulse += (trimval - 40);
  uplidpulse = constrain(uplidpulse, 280, 500);
  lolidpulse = map(yval, 0, 1023, 410, 280);
  lolidpulse += (trimval / 2);
  lolidpulse = constrain(lolidpulse, 280, 500);

  pwm.setPWM(0, 0, lexpulse);
  pwm.setPWM(1, 0, leypulse);
  pwm.setPWM(2, 0, rexpulse);
  pwm.setPWM(3, 0, reypulse); 

  // Random blinking with consistent duration
  if (!isBlinking && (random(1000) > 995 || switchval == HIGH)) {
    isBlinking = true;
    blinkStartTime = millis();
  }

  if (isBlinking) {
    if (millis() - blinkStartTime < blinkDuration) {
      blink();
    } else {
      isBlinking = false;
    }
  } else {
    // If not blinking, set the eyelids to follow normal eye movements
    pwm.setPWM(4, 0, uplidpulse); // Upper eyelid back to normal
    pwm.setPWM(5, 0, lolidpulse); // Lower eyelid back to normal
  }

  Serial.println(lexpulse);
  
  delay(5);
}
